using LatticeGPU
using ADerrors
using BDIO


function read_ff(name::String)

    file = BDIO_open(name,"r","FerFlow correlators")

    while BDIO_get_uinfo(file) != 1
        BDIO_seek!(file)
    end
    dims=Vector{Int32}(undef,1)
    iL=Vector{Int32}(undef,4)
    dpars=Vector{Float64}(undef,3) # m0,csw,Cfl
    th=Vector{ComplexF64}(undef,4)
    nflow=Vector{Int32}(undef,1)
    N_noise=Vector{Int32}(undef,1)

    BDIO_read(file, dims)
    BDIO_read(file, iL)
    BDIO_read(file, dpars)
    BDIO_read(file, th)
    BDIO_read(file, nflow)
    flow_times=Vector{Float64}(undef,nflow)
    BDIO_read(file, flow_times)
    BDIO_read(file, N_noise)

    N_noise = N_noise[1]
    nflow = nflow[1]
    TvecR = Vector{Float64}(undef,iL[4]);
    TvecC = Vector{ComplexF64}(undef,iL[4]);
    CN = Vector{ComplexF64}(undef,1);

    pp_corr = Array{Float64}(undef,1,iL[4],N_noise);
    ap_corr = Array{ComplexF64}(undef,1,iL[4],N_noise);
    pp_corr_t = Array{Float64}(undef,1,iL[4],N_noise,nflow);
    ap_corr_t = Array{ComplexF64}(undef,1,iL[4],N_noise,nflow);
    Sigma = Array{ComplexF64}(undef,1,N_noise,nflow);

    pp_corr_mc = Array{Float64}(undef,0,iL[4],N_noise);
    ap_corr_mc = Array{ComplexF64}(undef,0,iL[4],N_noise);
    pp_corr_t_mc = Array{Float64}(undef,0,iL[4],N_noise,nflow);
    ap_corr_t_mc = Array{ComplexF64}(undef,0,iL[4],N_noise,nflow);
    Sigma_mc = Array{ComplexF64}(undef,0,N_noise,nflow);

    while BDIO_get_uinfo(file) != 8
        BDIO_seek!(file)
    end

    while BDIO_get_uinfo(file) == 8

        for noi in 1:N_noise
            BDIO_read(file,TvecR)
            pp_corr[1,:,noi] .= TvecR

            BDIO_read(file,TvecC)
            ap_corr[1,:,noi] .= TvecC

            for fl in 1:nflow
                BDIO_read(file,TvecR)
                pp_corr_t[1,:,noi,fl].= TvecR

                BDIO_read(file,TvecC)
                ap_corr_t[1,:,noi,fl].= TvecC

                BDIO_read(file,CN)
                Sigma[1,noi,fl]= CN[1]
            end
        end

        pp_corr_mc=[pp_corr_mc;pp_corr]
        ap_corr_mc=[ap_corr_mc;ap_corr]
        pp_corr_t_mc=[pp_corr_t_mc;pp_corr_t]
        ap_corr_t_mc=[ap_corr_t_mc;ap_corr_t]
        Sigma_mc=[Sigma_mc;Sigma]

        BDIO_seek!(file,2)
    end

    BDIO_close!(file)

    return pp_corr_mc,ap_corr_mc,pp_corr_t_mc,ap_corr_t_mc,Sigma_mc
end

